//! Function to handle Discord API connection.

use crate::error::{Result, SkadiError};
use log::info;
use serenity::{
    all::{EventHandler, GatewayIntents},
    async_trait,
    model::prelude::Ready,
    prelude::Context,
    Client,
};
use std::env;

/// The EventHandler provided to the Serenity crate to handle Discord events.
struct DiscordEventHandler;

#[async_trait]
impl EventHandler for DiscordEventHandler {
    /// Handler called when the bot successfully connected to the Discord API.
    async fn ready(&self, _: Context, data_about_bot: Ready) {
        info!(
            "Successfully connected to Discord as {}#{}.",
            data_about_bot.user.name,
            data_about_bot.user.discriminator.unwrap()
        );
    }
}

/// Start a Discord session.
/// The `DISCORD_TOKEN` environment variable should be filled with the Discord-provided application token.
/// The connection might failed if the `DISCORD_TOKEN` environment variable is not set or if the connection
/// to the Discord API failed.
pub async fn connect_discord() -> Result<()> {
    let discord_token = env::var("DISCORD_TOKEN")
        .map_err(|err| SkadiError::DiscordConnectionError(err.to_string()))?;

    let intents = GatewayIntents::empty();
    let mut client = Client::builder(discord_token, intents)
        .event_handler(DiscordEventHandler)
        .await
        .map_err(|err| SkadiError::DiscordConnectionError(err.to_string()))?;

    if let Err(err) = client.start().await {
        Err(SkadiError::DiscordConnectionError(err.to_string()))?;
    };

    Ok(())
}
