//! Matrix utilities to connect to the homeserver API.

use crate::error::{Result, SkadiError};
use log::info;
use matrix_sdk::{config::SyncSettings, ruma::UserId, Client};
use std::env;

/// The bot identifier localpart on the homeserver.
const MATRIX_USER_ID: &str = "@skadi";

/// Start a Matrix session.
/// A user account for the bot should be created beforehand on the homeserver with the localpart [`MATRIX_USER_ID`].
/// The environment variables `MATRIX_SERVER` and `MATRIX_SECRET` should be set with the homeserver address and the bot user passowrd respectively.
/// The connnection might failed if any of the `MATRIX_SERVER` and `MATRIX_SECRET` is not set, or if the bot failed to connect to the Matrix homeserver.
pub async fn connect_matrix() -> Result<()> {
    let server_id = env::var("MATRIX_SERVER")
        .map_err(|err| SkadiError::MatrixConnectionError(err.to_string()))?;

    let user_id = UserId::parse(format!("{MATRIX_USER_ID}:{server_id}"))
        .map_err(|err| SkadiError::MatrixConnectionError(err.to_string()))?;

    let password = env::var("MATRIX_SECRET")
        .map_err(|err| SkadiError::MatrixConnectionError(err.to_string()))?;

    let client = Client::builder()
        .server_name(user_id.server_name())
        .build()
        .await
        .map_err(|err| SkadiError::MatrixConnectionError(err.to_string()))?;

    client
        .matrix_auth()
        .login_username(user_id, &password)
        .send()
        .await
        .map_err(|err| SkadiError::MatrixConnectionError(err.to_string()))?;

    // We have to sync one before launching the sync loop as it will prevent the bot from
    // reacting to the old messages.
    let response = client
        .sync_once(SyncSettings::new())
        .await
        .map_err(|err| SkadiError::MatrixConnectionError(err.to_string()))?;

    info!("Sucessfully connected to Matrix homeserver as {MATRIX_USER_ID}:{server_id}");

    client
        .sync(SyncSettings::default().token(response.next_batch))
        .await
        .map_err(|err| SkadiError::MatrixConnectionError(err.to_string()))?;

    Ok(())
}
